/*
 * $Id: socket.h,v 3.1 1994/01/24 16:06:03 too Exp $
 *
 * Copyright (c) 1993 AmiTCP/IP Group, <amitcp-group@hut.fi>
 *                    Helsinki University of Technology, Finland.
 *                    All rights reserved.
 *
 * HISTORY
 * $Log: socket.h,v $
 * Revision 3.1  1994/01/24  16:06:03  too
 * Added definition of `struct iovec'
 *
 * Revision 1.11  1993/07/07  14:55:52  too
 * Removed inclusion of CLIB_SOCKET_PROTOS_H
 *
 * Revision 1.10  1993/06/04  11:16:15  jraja
 * Fixes for first public release.
 *
 * Revision 1.9  1993/06/01  08:12:25  jraja
 * Added definition UNIQUE_ID to be used with Release(CopyOf)Socket().
 *
 * Revision 1.6  1993/04/26  11:22:59  jraja
 * Updated to include socket protos/pragmas/inlines (if not in KERNEL).
 *
 * Revision 1.5  93/04/13  21:59:15  21:59:15  jraja (Jarno Tapio Rajahalme)
 * Added inline/prototype for Select() function (#ifndef KERNEL).
 * 
 * Revision 1.4  93/03/22  01:25:21  01:25:21  ppessi (Pekka Pessi)
 * An API version of socket.h
 * 
 * Revision 1.1  92/11/20  15:42:36  15:42:36  jraja (Jarno Tapio Rajahalme)
 * Initial revision
 *
 */

/*
 * Copyright (c) 1982,1985,1986,1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      @(#)socket.h    7.13 (Berkeley) 4/20/91
 */

/*
 * Definitions related to sockets: types, address families, options.
 */

/*
 * Definition for Release(CopyOf)Socket unique id
 */
#define UNIQUE_ID       -1

/*
 * Types
 */
#define SOCK_STREAM     1               /* stream socket */
#define SOCK_DGRAM      2               /* datagram socket */
#define SOCK_RAW        3               /* raw-protocol interface */
#define SOCK_RDM        4               /* reliably-delivered message */
#define SOCK_SEQPACKET  5               /* sequenced packet stream */

/*
 * Option flags per-socket.
 */
#define SO_DEBUG        $0001          /* turn on debugging info recording */
#define SO_ACCEPTCONN   $0002          /* socket has had listen() */
#define SO_REUSEADDR    $0004          /* allow local address reuse */
#define SO_KEEPALIVE    $0008          /* keep connections alive */
#define SO_DONTROUTE    $0010          /* just use interface addresses */
#define SO_BROADCAST    $0020          /* permit sending of broadcast msgs */
#define SO_USELOOPBACK  $0040          /* bypass hardware when possible */
#define SO_LINGER       $0080          /* linger on close if data present */
#define SO_OOBINLINE    $0100          /* leave received OOB data in line */

/*
 * Additional options, not kept in so_options.
 */
#define SO_SNDBUF       $1001          /* send buffer size */
#define SO_RCVBUF       $1002          /* receive buffer size */
#define SO_SNDLOWAT     $1003          /* send low-water mark */
#define SO_RCVLOWAT     $1004          /* receive low-water mark */
#define SO_SNDTIMEO     $1005          /* send timeout */
#define SO_RCVTIMEO     $1006          /* receive timeout */
#define SO_ERROR        $1007          /* get error status and clear */
#define SO_TYPE         $1008          /* get socket type */

/*
 * Structure used for manipulating linger option.
 */
OBJECT linger
    l_onoff:LONG,               /* option on/off */
    l_linger:LONG               /* linger time */

/*
 * Level number for (get/set)sockopt() to apply to socket itself.
 */
#define SOL_SOCKET      $ffff          /* options for socket level */

/*
 * Address families.
 */
#define AF_UNSPEC       0               /* unspecified */
#define AF_UNIX         1               /* local to host (pipes, portals) */
#define AF_INET         2               /* internetwork: UDP, TCP, etc. */
#define AF_IMPLINK      3               /* arpanet imp addresses */
#define AF_PUP          4               /* pup protocols: e.g. BSP */
#define AF_CHAOS        5               /* mit CHAOS protocols */
#define AF_NS           6               /* XEROX NS protocols */
#define AF_ISO          7               /* ISO protocols */
#define AF_OSI          AF_ISO
#define AF_ECMA         8               /* european computer manufacturers */
#define AF_DATAKIT      9               /* datakit protocols */
#define AF_CCITT        10              /* CCITT protocols, X.25 etc */
#define AF_SNA          11              /* IBM SNA */
#define AF_DECnet       12              /* DECnet */
#define AF_DLI          13              /* DEC Direct data link interface */
#define AF_LAT          14              /* LAT */
#define AF_HYLINK       15              /* NSC Hyperchannel */
#define AF_APPLETALK    16              /* Apple Talk */
#define AF_ROUTE        17              /* Internal Routing Protocol */
#define AF_LINK         18              /* Link layer interface */
#define pseudo_AF_XTP   19              /* eXpress Transfer Protocol (no AF) */

#define AF_MAX          20

/*
 * Structure used by kernel to store most
 * addresses.
 */
OBJECT sockaddr
    sa_len:UBYTE,                 /* total length */
    sa_family:UBYTE,              /* address family */
    sa_data[14]:BYTE             /* actually longer; address value */

/*
 * Structure used by kernel to pass protocol
 * information in raw sockets.
 */
OBJECT sockproto
    sp_family:UWORD,              /* address family */
    sp_protocol:UWORD            /* protocol */

/*
 * Protocol families, same as address families for now.
 */

#define PF_UNSPEC       AF_UNSPEC
#define PF_UNIX         AF_UNIX
#define PF_INET         AF_INET
#define PF_IMPLINK      AF_IMPLINK
#define PF_PUP          AF_PUP
#define PF_CHAOS        AF_CHAOS
#define PF_NS           AF_NS
#define PF_ISO          AF_ISO
#define PF_OSI          AF_ISO
#define PF_ECMA         AF_ECMA
#define PF_DATAKIT      AF_DATAKIT
#define PF_CCITT        AF_CCITT
#define PF_SNA          AF_SNA
#define PF_DECnet       AF_DECnet
#define PF_DLI          AF_DLI
#define PF_LAT          AF_LAT
#define PF_HYLINK       AF_HYLINK
#define PF_APPLETALK    AF_APPLETALK
#define PF_ROUTE        AF_ROUTE
#define PF_LINK         AF_LINK
#define PF_XTP          pseudo_AF_XTP   /* really just proto family, no AF */

#define PF_MAX          AF_MAX

/*
 * Maximum queue length specifiable by listen.
 */
#define SOMAXCONN       5

/*
 * Message header for recvmsg and sendmsg calls.
 * Used value-result for recvmsg, value only for sendmsg.
 */
OBJECT iovec
        iov_base:PTR TO BYTE,
        iov_len:LONG

OBJECT msghdr
    msg_name:PTR TO BYTE,               /* optional address */
    msg_namelen:ULONG,            /* size of address */
    msg_iov:PTR TO iovec,         /* scatter/gather array */
    msg_iovlen:ULONG,             /* # elements in msg_iov */
    msg_control:PTR TO BYTE,            /* ancillary data, see below */
    msg_controllen:ULONG,         /* ancillary data buffer len */
    msg_flags:LONG              /* flags on received message */


#define MSG_OOB         $1             /* process out-of-band data */
#define MSG_PEEK        $2             /* peek at incoming message */
#define MSG_DONTROUTE   $4             /* send without using routing tables */
#define MSG_EOR         $8             /* data completes record */
#define MSG_TRUNC       $10            /* data discarded before delivery */
#define MSG_CTRUNC      $20            /* control data lost before delivery */
#define MSG_WAITALL     $40            /* wait for full request or error */

/*
 * Header for ancillary data objects in msg_control buffer.
 * Used for additional information with/about a datagram
 * not expressible by flags.  The format is a sequence
 * of message elements headed by cmsghdr structures.
 */
OBJECT cmsghdr
    cmsg_len:ULONG,               /* data byte count, including hdr */
    cmsg_level:LONG,             /* originating protocol */
    cmsg_type:LONG              /* protocol-specific type */
/* followed by  u_char  cmsg_data[]; */

/* given pointer to struct adatahdr, return pointer to data */
#define CMSG_DATA(cmsg)         ((cmsg) + 1)

/* given pointer to struct adatahdr, return pointer to next adatahdr */
/* --- got to do ---
#define CMSG_NXTHDR(mhdr, cmsg) \
        (((caddr_t)(cmsg) + (cmsg).cmsg_len + sizeof(struct cmsghdr) > \
            (mhdr)->msg_control + (mhdr)->msg_controllen) ? \
            (struct cmsghdr *)NULL : \
            (struct cmsghdr *)((caddr_t)(cmsg) + ALIGN((cmsg)->cmsg_len)))
*/

#define CMSG_FIRSTHDR(mhdr)     ((mhdr).msg_control)

/* "Socket"-level control message types: */
#define SCM_RIGHTS      $01            /* access rights (array of int) */

/*
 * 4.3 compat sockaddr, move to compat file later
 */
OBJECT osockaddr
    sa_family:UWORD,              /* address family */
    sa_data[14]:BYTE            /* up to 14 bytes of direct address */

/*
 * 4.3-compat message header (move to compat file later).
 */
OBJECT omsghdr
    msg_name:PTR TO BYTE,        /* optional address */
    msg_namelen:LONG,            /* size of address */
    msg_iov:PTR TO iovec,        /* scatter/gather array */
    msg_iovlen:LONG,             /* # elements in msg_iov */
    msg_accrights:PTR TO BYTE,   /* access rights sent/received */
    msg_accrightslen:LONG



